@@ -43,6 +43,7 @@
 #ifdef HAVE_COGL_GLES2
 #include "cogl-pipeline-progend-glsl-private.h"
 #endif
+#include "cogl-private.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -535,7 +536,7 @@ enable_gl_state (CoglDrawFlags flags,
     }
 
   if (G_UNLIKELY (ctx->legacy_state_set) &&
-      (flags & COGL_DRAW_SKIP_LEGACY_STATE) == 0)
+      _cogl_get_enable_legacy_state ())
     {
       /* If we haven't already created a derived pipeline... */
       if (!copy)
@@ -548,9 +549,6 @@ enable_gl_state (CoglDrawFlags flags,
 
   _cogl_pipeline_flush_gl_state (source, skip_gl_color, n_tex_coord_attribs);
 
-  if (ctx->enable_backface_culling)
-    enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
-
   _cogl_bitmask_clear_all (&ctx->temp_bitmask);
 
   /* Bind the attribute pointers. We need to do this after the
@@ -725,7 +723,6 @@ enable_gl_state (CoglDrawFlags flags,
   set_enabled_arrays (&ctx->arrays_enabled, &ctx->temp_bitmask);
 
   _cogl_enable (enable_flags);
-  _cogl_flush_face_winding ();
 
   return source;
 }
@@ -1015,7 +1012,7 @@ draw_wireframe (CoglVerticesMode mode,
                                   0x00, 0xff, 0x00, 0xff);
     }
 
-  cogl_push_source (wire_pipeline);
+  _cogl_push_source (wire_pipeline, FALSE);
 
   /* temporarily disable the wireframe to avoid recursion! */
   COGL_DEBUG_CLEAR_FLAG (COGL_DEBUG_WIREFRAME);
@@ -1026,8 +1023,7 @@ draw_wireframe (CoglVerticesMode mode,
                          1,
                          COGL_DRAW_SKIP_JOURNAL_FLUSH |
                          COGL_DRAW_SKIP_PIPELINE_VALIDATION |
-                         COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH |
-                         COGL_DRAW_SKIP_LEGACY_STATE);
+                         COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH);
 
   COGL_DEBUG_SET_FLAG (COGL_DEBUG_WIREFRAME);
 
